"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.independencyBinCommand = void 0;
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var ora_1 = tslib_1.__importDefault(require("ora"));
var fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
var package_const_1 = require("../../shared/constants/package.const");
var utils_1 = require("../../shared/utils");
var messages_1 = require("../../shared/messages");
var independency_helpers_1 = require("./independency.helpers");
exports.independencyBinCommand = function (prog) {
    prog
        .command('migration independency')
        .describe('Add all necessary settings to the package so that it can function outside of the workspace')
        .example('migration independency')
        .action(function () { return tslib_1.__awaiter(void 0, void 0, void 0, function () {
        var bootSpinner, packageDir, packageJsonPath, tsconfigJsonPath, packageJson, tsconfigJson, err_1;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    bootSpinner = ora_1.default(messages_1.independencyMessage.generating());
                    return [4 /*yield*/, utils_1.findWorkspacePackageDir()];
                case 1:
                    packageDir = _a.sent();
                    packageJsonPath = path_1.default.resolve(packageDir, package_const_1.PACKAGE_JSON);
                    tsconfigJsonPath = path_1.default.resolve(packageDir, package_const_1.TSCONFIG_JSON);
                    return [4 /*yield*/, fs_extra_1.default.readJSON(packageJsonPath)];
                case 2:
                    packageJson = (_a.sent());
                    return [4 /*yield*/, fs_extra_1.default.readJSON(tsconfigJsonPath)];
                case 3:
                    tsconfigJson = (_a.sent());
                    bootSpinner.start();
                    _a.label = 4;
                case 4:
                    _a.trys.push([4, 9, , 10]);
                    return [4 /*yield*/, independency_helpers_1.makeTsconfigJsonIndependent(tsconfigJsonPath, tsconfigJson)];
                case 5:
                    _a.sent();
                    return [4 /*yield*/, independency_helpers_1.makePackageJsonIndependent(packageJsonPath, packageJson)];
                case 6:
                    _a.sent();
                    return [4 /*yield*/, independency_helpers_1.copyIndependencyTemplate(packageDir)];
                case 7:
                    _a.sent();
                    return [4 /*yield*/, utils_1.installDependencies()];
                case 8:
                    _a.sent();
                    bootSpinner.succeed(messages_1.independencyMessage.successful());
                    return [3 /*break*/, 10];
                case 9:
                    err_1 = _a.sent();
                    bootSpinner.fail(messages_1.independencyMessage.failed());
                    utils_1.logError(err_1);
                    process.exit(1);
                    return [3 /*break*/, 10];
                case 10: return [2 /*return*/];
            }
        });
    }); });
};
